// crates/ih-muse/src/tasks/mod.rs

mod cluster_monitor;
mod element_registration;
mod flush_task;
mod init_task;
mod metric_sender;

pub use cluster_monitor::start_cluster_monitor;
pub use element_registration::start_element_registration_task;
pub use flush_task::start_recorder_flush_task;
pub use init_task::start_init_task;
pub use metric_sender::start_metric_sender_task;
