// crates/ih-muse-proto/src/lib.rs

mod cluster_state;
mod element;
mod element_kind;
mod metric;
mod node_elem_ranges;
pub mod prelude;
mod timestamp_resolution;
pub mod types;
mod utils;

pub use cluster_state::{NodeInfo, NodeState, NodeStatus};
pub use element::{generate_local_element_id, ElementRegistration, NewElementsResponse};
pub use element_kind::ElementKindRegistration;
pub use metric::{metric_id_from_code, MetricDefinition, MetricPayload, MetricQuery};
pub use node_elem_ranges::{GetRangesRequest, NodeElementRange, OrdRangeInc};
pub use timestamp_resolution::TimestampResolution;
pub use types::{ElementId, ElementKindId, LocalElementId, MetricId, MetricValue, Timestamp};
