# ih_muse/exceptions.pyi

class MuseError(Exception): ...
class MuseInitializationTimeoutError(MuseError): ...
class ConfigurationError(MuseError): ...
class ClientError(MuseError): ...
class RecordingError(MuseError): ...
class ReplayingError(MuseError): ...
class InvalidFileExtensionError(MuseError): ...
class InvalidElementKindCodeError(MuseError): ...
class InvalidMetricCodeError(MuseError): ...
class DurationConversionError(MuseError): ...
class PanicException(MuseError): ...
class OtherPythonErrors(MuseError): ...
